/* gmn2midi */
/* version 1.0, 98/10/18 */
/* (c) 1998 by Ludger Martin and Holger H. Hoos */

/* GUIDO to MIDI converter */
/* based on the GUIDO/NoteAbility parser interface */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <stl.h>

#include <MotorolaFileIO.hpp>
#include <TextFileIO.hpp>
#include <Exception.hpp>

#include <Guido2Midi.hpp>

int main(int argc,const char*argv[])
{
  // print Copyright
  printf("\nGMN2MIDI v1.0 \n\n");
  printf("written 1998 by Ludger Martin and Holger H. Hoos ");
  printf("as part of the GUIDO toolbox\n");
  printf("for more information on GUIDO Music Notation please visit\n");
  printf("http://www.informatik.tu-darmstadt.de/AFS/GUIDO\n\n");

  // if there are no parameter or -h, print help
  if((argc <= 1) ? true :
                   (strcmp(argv[1],"-h")==0 || strcmp(argv[1],"/h")==0||
                    strcmp(argv[1],"-help")==0 || strcmp(argv[1],"/help")==0||
                    strcmp(argv[1],"-?")==0 || strcmp(argv[1],"/?")==0 ))
    {
      printf ("description:\n");
      printf ("  converts GMN (guido music notation) files to MIDI\n\n");
      printf ("call syntax:\n");
      printf ("  gmn2midi [options] <filename>\n");
      printf ("  gmn2midi [options] <filename>.gmn\n");
      printf ("  -> creates file <filename>.mid from <filename>.gmn\n");
      printf ("\n");
      printf (" options: -help                 -> displays this information\n");
      printf ("          -l<logfile>           -> set logfile (stderr)\n");
      printf ("          -c<configfile>        -> set configfile\n");
      printf ("          -0                    -> save as MIDI format 0 \n");
      printf ("                                   intead of MIDI format 1 \n");
      printf ("          -s<register>          -> set register\n");

      printf ("            q<quater per minute> - default tempo (100)\n");
      printf ("            t<ticks per quater>  - ticks per quater for delay (64*9*5)\n");
      printf ("            i<intensity>         - default intensity [0.0 ... 1.0] (0.8)\n");
      printf ("            c<use>,<channel>     - use: t - uese channel \n");
      printf ("                                        f - dont uese channel (10) \n");
      printf ("            m                    - allow multipel tempi \n");
      printf ("            d<name>,<intensity>  - set dynamic [any text],[0.0 ... 1.0] \n");
      printf ("            f<factor>            - factor to multiply for default (0.8)\n");
      printf ("            s<factor>            - factor to multiply for staccato (0.5)\n");
      printf ("            l<factor>            - factor to multiply for slur (1.0)\n");
      printf ("            e<factor>            - factor to multiply for tenuto (0.95)\n");
      printf ("            r<factor>            - factor to multiply for fermata (2.0)\n");
      printf ("            a<factor>            - factor to multiply for accent (1.1)\n");
      printf ("            z<factor>            - factor to multiply for marcato (1.2)\n");
      printf ("\n");
      return (0);
    }

  // read parameter and filename
  int i;
  unsigned short quarterPerMinute = 100;
  unsigned short ticksPerQuarter = 64*9*5;
  float intensity = 0.8;
  bool format0 = false;
  bool useChannel[16] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1};
  bool multipleTempo = false;
  FileIO *logFile = new TextFileIO (TextFileIO::stdErr); // set logFile to stderr
  deque<MString> argQueue;
  DynamicList dynamicList;
  float factorDefault = 0.8;
  float factorStaccato = 0.5;
  float factorTenuto = 0.95;
  float factorSlur = 1.0;
  float factorFermata = 2.0;
  float factorAccent = 1.1;
  float factorMarcato = 1.2;
  bool readDefConfigFile = true;
  
  // write args in Queue
  for (i = 1; i < argc - 1; i++)
    {
      argQueue.push_back (MString(argv[i]));
      // check if configfile-option -c is mentioned
      if ((argv[i][1] == 'c')||(argv[i][1] == 'C'))
        readDefConfigFile = false;
    }

  // read configfile 'gmn2midi.cfg' if there is no -c option
  if (readDefConfigFile)
    {
      // open Configfile if present
      TextFileIO configFile;
      try 
	{
	  configFile.open ("gmn2midi.cfg", FileIO::readOnly);
	  printf ("reading configfile: gmn2midi.cfg\n");
	  // attention: read the configfile befove the oter args
	  MString string;
	  try
	    {
	      while (true)
		{
		  configFile >> string;
		  argQueue.push_front (string);
		}
	    }
	  catch (...) {  }
        }
      catch (...)
	{
	  printf ("no configfile 'gmn2midi.cfg' fond!\n");
        }
    }
  
  // read Queue and
  for (;!argQueue.empty (); argQueue.pop_front ())
    // Check for coment
    if (argQueue.front ()[0] != '%')
      switch (argQueue.front ()[1])
	{
          case 'l':
          case 'L':
	       // set logFile
	       delete logFile;
               logFile = new TextFileIO (&(argQueue.front ()[2]), FileIO::writeOnly);
               break;
          case 'c':
          case 'C':
               {
		 try
		   {
		     // read configfile and write into arqQueue
		     printf ("reading configfile: %s\n", &(argQueue.front ()[2]));
		     TextFileIO configFile (&(argQueue.front ()[2]), FileIO::readOnly);
		     // attention: read the configfile befove the oter args
		     // but delete the configfile option
		     argQueue.pop_front ();
		     MString string;
		     try
		       {
			 while (true)
			   {
			     configFile >> string;
			     argQueue.push_front (string);
			   }
		       }
		     catch (...) {  }
		     // we have to add a dummy arg because we deleted the
		     // configfile option
		     argQueue.push_front (MString ("delete me"));
		   }
		 catch (...)
		   {
		     printf ("Error while reading configfile\n");
		     exit (1);
		   }
	       }
	       break;
          case '0':
               format0 = true;
               break;
          case 's':
          case 'S':
               switch (argQueue.front ()[2])
                 {
                   case 'q':
                   case 'Q':
                        quarterPerMinute = MString (&(argQueue.front ()[3])).asUnsigned ();
                        break;
                   case 't':
                   case 'T':
                        ticksPerQuarter = MString (&(argQueue.front ()[3])).asUnsigned ();
                        break;
                   case 'i':
                   case 'I':
                        intensity = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   case 'c':
                   case 'C':
                        if (argQueue.front ()[3] == 't')
                          useChannel[MString (&(argQueue.front ()[5])).asUnsigned ()] = 1;
                        else
                          useChannel[MString (&(argQueue.front ()[5])).asUnsigned ()] = 0;
                        break;
                   case 'm':
                   case 'M':
                        multipleTempo = true;
                        break;
                   case 'd':
                   case 'D':
                        {
			  // store dynamic-name and intensity in dynamicList;
			  Dynamic dynamic;
			  dynamic.name = MString (&(argQueue.front ()[3]));
			  dynamic.name.remove (dynamic.name.indexOf (','));
			  dynamic.intensity = MString(&(argQueue.front ()
			    [argQueue.front ().indexOf (',') + 1])).asDouble ();
			  dynamicList.push_back (dynamic);
			}
                        break;
                   case 'f':
                   case 'F':
                        factorDefault = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   case 's':
                   case 'S':
                        factorStaccato = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   case 'l':
                   case 'L':
                        factorSlur = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   case 'e':
                   case 'E':
                        factorTenuto = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   case 'r':
                   case 'R':
                        factorFermata = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   case 'a':
                   case 'A':
                        factorAccent = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   case 'z':
                   case 'Z':
                        factorMarcato = MString (&(argQueue.front ()[3])).asDouble ();
                        break;
                   default:
                        // unknown option
                        printf ("unknown option: %s\n", (char*)argQueue.front ());
                        break;
                 }
               break;
          default:
               // unknown option
               printf ("unknown option: %s\n", (char*)argQueue.front ());
               break;
        }

  // prepare filenames
  MString inFileName (argv[argc - 1]);
  MString outFileName (argv[argc - 1]);

  if (outFileName.lastIndexOf (".gmn") == 0)
    inFileName += MString (".gmn");
  else
    outFileName.remove (outFileName.lastIndexOf (".gmn"));
  outFileName += MString (".mid");

  printf("converting '%s' to '%s' ... \n",
         (char*)inFileName, (char*)outFileName);

  // load Guido-File
  Guido guido (inFileName);

  // convert Guido to Midi and save the Midi
  try
    {
      // convert Guido to Midi
      Midi midi = guido.guido2Midi (ticksPerQuarter, quarterPerMinute,
                                    intensity, useChannel, multipleTempo,
                                    logFile, dynamicList, factorDefault, 
				    factorStaccato, factorTenuto, factorSlur,
				    factorFermata, factorAccent, factorMarcato);
      // convert to MIDI format 0;
      if (format0)
        {
          printf ("converting to format 0 \n");
          midi.convertToType0 ();
        }
      // save midi
     MotorolaFileIO midiFile (outFileName, FileIO::writeOnly);
     midiFile << midi;
    }
  catch (Exception &exc)
    {
      printf ("Error while processing (ID: %lu) \n", exc.id ());
    }

  // close logFile
  delete logFile;

  // done
  printf("... done.\n\n");
  return(0);
}

