/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: FileIO.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     FileIO                                                                   *
*******************************************************************************/
#ifndef _FILEIO_
#define _FILEIO_

#include <MString.hpp>
#include <IO.hpp>

// Fehlernummern und Meldungen
#define ERROR_FILE_OPEN                  1
#define ERROR_FILE_WRITE                 2
#define ERROR_FILE_READ                  3
#define ERROR_FILE_EOF                   4
#define ERROR_FILE_NOT_OPEN              5
#define ERROR_FILE_SEEK                  6
#define ERROR_FILE_TELL                  7

class FileIO : public IO
{
public:
  enum OpenMode {readOnly, writeOnly, append, readWrite, writeRead, appendRead};

  // ffnen und schlieen einer Datei
  virtual void _Export open (const MString& name, const OpenMode mode) = 0;
  virtual void _Export close () = 0;

  // Schreiben in eine Datei
  virtual FileIO& _Export operator<< (const unsigned char c) = 0;
  virtual FileIO& _Export operator<< (const signed char c) = 0;
  virtual FileIO& _Export operator<< (const unsigned short s) = 0;
  virtual FileIO& _Export operator<< (const signed short s) = 0;
  virtual FileIO& _Export operator<< (const unsigned long l) = 0;
  virtual FileIO& _Export operator<< (const signed long l) = 0;
  virtual FileIO& _Export operator<< (const MString& string) = 0;

  // Lesen aus einer Datei
  virtual FileIO& _Export operator>> (unsigned char& c) = 0;
  virtual FileIO& _Export operator>> (signed char& c) = 0;
  virtual FileIO& _Export operator>> (unsigned short& s) = 0;
  virtual FileIO& _Export operator>> (signed short& s) = 0;
  virtual FileIO& _Export operator>> (unsigned long& l) = 0;
  virtual FileIO& _Export operator>> (signed long& l) = 0;
  virtual FileIO& _Export operator>> (MString& string) = 0;

  // positionieren
  virtual void _Export seek (const long int position) = 0;
  virtual long int _Export tell (void) = 0;
};

#endif

