/*******************************************************************************
* FILE NAME: FindFile.cpp                                                      * 
*                                                                              * 
* DESCRIPTION:                                                                 * 
*   Class implementation of the class:                                         * 
*     FindeFile                                                                * 
*******************************************************************************/ 
#if __OS2__ 
#define INCL_DOSFILEMGR 
#define INCL_DOSERRORS 
#include <os2.h> 
#elif __WINDOWS__ 
#include <windows.h> 
#else // Unix
#include <dirent.h> 
#include <sys/stat.h>
#include <fnmatch.h>
#endif 
 
#include "FindFile.hpp" 
#include "Exception.hpp" 
 
// Nchste Datei finden 
MString FindFile::next () 
{ 
#if __OS2__ 
  static HDIR hdirFindHandle; 
  static FILEFINDBUF3 findBuffer; 
  ULONG ulFindCount = 1; 
 
  // fertig mit Suche 
  if (schliessen) 
    { 
      DosFindClose (hdirFindHandle); 
      return (MString ()); 
    } 
 
  if (findFirst) 
    { 
      // ersten Eintrag finden 
      findFirst = false; 
      hdirFindHandle = HDIR_SYSTEM; 
      findBuffer.oNextEntryOffset = 0; 
 
      if (DosFindFirst (verzeichnis + "//" + maske, &hdirFindHandle, FILE_NORMAL, 
                    &findBuffer, sizeof (FILEFINDBUF3), 
                    &ulFindCount, FIL_STANDARD)) 
        throw (Exception (ERROR_NO_MORE_FILES_FOUND)); 
    } 
  else 
    { 
      // weiteren Eintrag finden 
      if (DosFindNext (hdirFindHandle, &findBuffer, 
                     sizeof (FILEFINDBUF3), &ulFindCount)) 
        throw (Exception (ERROR_NO_MORE_FILES_FOUND)); 
    } 
 
  return (MString (findBuffer.achName)); 
 
#elif __WINDOWS__ 
  static WIN32_FIND_DATA findData; 
  static HANDLE findHandle; 
 
  // fertig mit Suche 
  if (schliessen) 
    { 
      FindClose (findHandle); 
      return (MString ()); 
    } 
 
  if (findFirst) 
    { 
      // ersten Eintrag finden 
      findFirst = false; 
 
      findData.dwFileAttributes = FILE_ATTRIBUTE_NORMAL; 
 
      if ((findHandle = FindFirstFile (verzeichnis + "//" + maske, 
                                       &findData)) == INVALID_HANDLE_VALUE) 
        throw (Exception (ERROR_NO_MORE_FILES_FOUND)); 
    } 
  else 
    { 
      // weiteren Eintrag finden 
      if (FindNextFile (findHandle, &findData) == false) 
        throw (Exception (ERROR_NO_MORE_FILES_FOUND)); 
    } 
 
  return (MString (findData.cFileName)); 
#else // Unix 
  // glob () benutzen siehe beispiel man 3 glob

  struct dirent **nameList;
  unsigned long anzahl = 0;
  unsigned long position = 0;

  // fertig mit Suche 
  if (schliessen) 
    { 
      if (anzahl != 0) 
        delete (nameList); 
      return (MString ("")); 
    } 

  if (findFirst) 
    { 
      // ersten Eintrag finden 
      findFirst = false; 

      anzahl = scandir (verzeichnis, &nameList, 0, alphasort);
 
      position = 0;
    }

  // weiteren Eintrag finden, d.h. Liste durchgehen 
  while (position < anzahl)
    {
      struct stat buf;

      if (stat (nameList[position]->d_name, &buf) == 0)
        if ((fnmatch(maske, nameList[position]->d_name, 0) == 0)&&
            S_ISREG (buf.st_mode))
          return (MString (nameList[position++]->d_name));

      ++position;
    }

  // Es wurde kein eintrag gefunden!
  throw (Exception (ERROR_NO_MORE_FILES_FOUND)); 
#endif 
} 

