/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: IntelFileIO.cpp                                                   *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     IntelFileIO                                                              *
*******************************************************************************/

#include "Exception.hpp"
#include "IntelFileIO.hpp"

// ffnen der Motoroladatei
void IntelFileIO::open (const MString& name, const OpenMode mode)
{
  switch (mode)
    {
      case readOnly:
           file = fopen (name, "rb");
           break;
      case writeOnly:
           file = fopen (name, "wb");
           break;
      case append:
           file = fopen (name, "ab");
           break;
      case readWrite:
           file = fopen (name, "r+b");
           break;
      case writeRead:
           file = fopen (name, "w+b");
           break;
      case appendRead:
           file = fopen (name, "a+b");
           break;
    }

  if (file == NULL)
    throw (Exception (ERROR_FILE_OPEN));

  gelesen = false;
}

// Schlieen der Motoroladatei
void IntelFileIO::close ()
{
  // Wenn File noch nicht geschlossen, schliessen
  if (file != NULL)
    fclose (file);
  file = NULL;
}

// schriebt ein uchar
IntelFileIO& IntelFileIO::operator<< (const unsigned char c)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  if (1 != fwrite (&c, 1, 1, file))
    throw (Exception (ERROR_FILE_WRITE));

  return *this;
}

// schreibt ein String mit Abschliessenden '\0'
IntelFileIO& IntelFileIO::operator<< (const MString& string)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  if (1 != fwrite ((const char*)string, string.length (), 1, file))
    throw (Exception (ERROR_FILE_WRITE));

  return *this << (unsigned char)'\0';
}

// liet ein uchar
IntelFileIO& IntelFileIO::operator>> (unsigned char& c)
{
  // Wenn Datei nicht geffnet, Fehler
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  // Wenn ein Char schon gelesen wurde, das zurckgeben
  if (gelesen == true)
    {
      c = cGelesen;
      gelesen = false;
      return *this;
    }

  // Zeichen lesen
  if (1 != fread (&c, 1, 1, file))
    if (feof (file))
      throw (Exception (ERROR_FILE_EOF));
    else
      throw (Exception (ERROR_FILE_READ));

  return *this;
}

// liet ein ushort
IntelFileIO& IntelFileIO::operator>> (unsigned short& s)
{
  unsigned char c1, c2;

  *this >> c1 >> c2;

  s = (c2 << 8) + c1;

  return *this;
}

// liet ein ulong
IntelFileIO& IntelFileIO::operator>> (unsigned long& l)
{
  unsigned short s1, s2;

  *this >> s1 >> s2;

  l = (s2 << 16) + s1;

  return *this;
}

// liest ein String mit Abschliessenden '\0'
IntelFileIO& IntelFileIO::operator>> (MString& string)
{
  unsigned long l = 0;
  do
    {
      *this >> (unsigned char&)string[l];
      l++;
    }
  while (string[l] != '\0');
  return *this;
}

// Erstes Zeichen angucken
unsigned char IntelFileIO::peek ()
{
  *this >> cGelesen;
  gelesen = true;
  return cGelesen;
}

// Position anspringen
void IntelFileIO::seek (const long int position)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  if (fseek (file, position, SEEK_SET) != 0)
    throw (Exception (ERROR_FILE_SEEK));
}

// Position holen
long int IntelFileIO::tell (void)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  long int position = ftell (file);
  if (position == -1)
    throw (Exception (ERROR_FILE_TELL));

  return (position);
}
