/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: IntelFileIO.hpp                                                   *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     IntelFileIO                                                              *
*******************************************************************************/
#ifndef _INTELFILEIO_
#define _INTELFILEIO_

#include <stdio.h>

#include <bool.h>
#include <FileIO.hpp>

class IntelFileIO : public FileIO
{
protected:
  FILE *file;
  bool gelesen;
  unsigned char cGelesen;

public:
  // Constructor / Destructor
  IntelFileIO ()
    {  file = NULL;  }
  IntelFileIO (const MString& name, const OpenMode mode)
    {  open (name, mode);  }
  virtual ~IntelFileIO ()
    {  close (); }

  // ffnen und schlieen einer Mididatei
  void _Export open (const MString& name, const OpenMode mode);
  void _Export close ();

  // Schreiben in eine Datei
  IntelFileIO& _Export operator<< (const unsigned char c);
  IntelFileIO& operator<< (const signed char c)
    {  return *this << (unsigned char)c;  }
  IntelFileIO& operator<< (const unsigned short s)
    {  return *this << (unsigned char)(s & 0xFF) << (unsigned char)(s >> 8);  }
  IntelFileIO& operator<< (const signed short s)
    {  return *this << (unsigned short)s;  }
  IntelFileIO& operator<< (const unsigned long l)
    {  return *this << (unsigned short)(l & 0xFFFF) << (unsigned short)(l >> 16);  }
  IntelFileIO& operator<< (const signed long l)
    {  return *this << (unsigned long)l;  }
  IntelFileIO& _Export operator<< (const MString& string);

  // Lesen aus einer Datei
  IntelFileIO& _Export operator>> (unsigned char& c);
  IntelFileIO& operator>> (signed char& c)
    {  return *this >> (unsigned char&)c;  }
  IntelFileIO& _Export operator>> (unsigned short& s);
  IntelFileIO& operator>> (signed short& s)
    {  return *this >> (unsigned short&)s;  }
  IntelFileIO& _Export operator>> (unsigned long& l);
  IntelFileIO& operator>> (signed long& l)
    {  return *this>> (unsigned long&)l;  }
  IntelFileIO& _Export operator>> (MString& string);

  // Erstes Zeichen angucken
  unsigned char _Export peek ();

  // positionieren
  void _Export seek (const long int position);
  long int _Export tell (void);
};

#endif
