/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MString.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MString                                                                  *
*******************************************************************************/

#include <MString.hpp>

// Constructor
MString::MString ()
{ 
  s = (char*) new char[1];
  s[0] = '\0';  
}

// Constructor
MString::MString (char c)
{ 
  s = (char*) new char[2];
  s[0] = c;
  s[1] = '\0';  
}

// Constructor
MString::MString (char* string)
{ 
  s = (char*) new char[strlen (string) + 1];
  strcpy (s, string);  
}

// Constructor
MString::MString (const char* string)
{ 
  s = (char*) new char[strlen (string) + 1];
  strcpy (s, string);  
}

// Constructor
MString::MString (const MString& string)
{ 
  s = (char*) new char[string.length () + 1];
  strcpy (s, (char*)string);  
}

// Constructor
MString::MString (const signed long l)
{  
  s = (char*) new char[256];
  sprintf (s, "%ld", l);  
}

// Constructor
MString::MString (const unsigned long l)
{  
  s = (char*) new char[256];
  sprintf (s, "%lu", l);  
}

// get first index of a searchString in a string
unsigned long MString::indexOf (const MString& searchString) const
{
  char *startPos = strstr (s, (char*)searchString);

  if (startPos != 0)
    return (startPos - s);

  return 0;
}

// get last index of a searchString in a string
unsigned long MString::lastIndexOf (const MString& searchString) const
{
  long startPos = length () -1;
  while (startPos)
    if (strstr(&(s[startPos]), (char*)searchString) != 0)
      return startPos;
    else
      --startPos;

  return 0;
}

// removes the substring beginning at index startPos
MString& MString::remove (unsigned long startPos)
{
  // prove startPos
  unsigned long len = length ();
  if ((startPos <= len)&&(len != 0))
    {
      // create new string, copy the contens until startpos - 1 and
      // delete the old
      char* oldS = s;
      s = (char*) new char[startPos + 1];

      strncpy (s, oldS, startPos);
      s[startPos] = '\0';

      delete[] oldS;
    }
  return *this;
}

MString& MString::operator+= (const MString& string)
{
  char* sOld = s;
  s = (char*) new char[string.length () + strlen (sOld) + 1];
  strcpy (s, sOld);
  strcpy (s + strlen (sOld), (char*) string);
  delete[] sOld;
  return *this;
}

MString MString::operator+ (const MString& string) const
{
  MString newString (*this);
  newString += string;
  return newString;
}

MString& MString::operator= (const MString& string)
{
  delete[] s;
  s = (char*) new char[string.length () + 1];
  strcpy (s, string.s);
  return *this;
}
