/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MotorolaFileIO.hpp                                                *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     MotorolaFileIO                                                           *
*******************************************************************************/
#ifndef _MOTOROLAFILEIO_
#define _MOTOROLAFILEIO_

#include <stdio.h>

#include <bool.h>
#include <FileIO.hpp>

class MotorolaFileIO : public FileIO
{
protected:
  FILE *file;
  bool gelesen;
  unsigned char cGelesen;

public:
  // Constructor / Destructor
  MotorolaFileIO ()
    {  file = NULL;  }
  MotorolaFileIO (const MString& name, const OpenMode mode)
    {  open (name, mode);  }
  virtual ~MotorolaFileIO ()
    {  close (); }

  // ffnen und schlieen einer Mididatei
  void _Export open (const MString& name, const OpenMode mode);
  void _Export close ();

  // Schreiben in eine Datei
  MotorolaFileIO& _Export operator<< (const unsigned char c);
  MotorolaFileIO& operator<< (const signed char c)
    {  return *this << (unsigned char)c;  }
  MotorolaFileIO& operator<< (const unsigned short s)
    {  return *this << (unsigned char)(s >> 8) << (unsigned char)(s & 0xFF);  }
  MotorolaFileIO& operator<< (const signed short s)
    {  return *this << (unsigned short)s;  }
  MotorolaFileIO& operator<< (const unsigned long l)
    {  return *this << (unsigned short)(l >> 16) << (unsigned short)(l & 0xFFFF);  }
  MotorolaFileIO& operator<< (const signed long l)
    {  return *this << (unsigned long)l;  }
  MotorolaFileIO& _Export operator<< (const MString& string);

  // Lesen aus einer Datei
  MotorolaFileIO& _Export operator>> (unsigned char& c);
  MotorolaFileIO& operator>> (signed char& c)
    {  return *this >> (unsigned char&)c;  }
  MotorolaFileIO& _Export operator>> (unsigned short& s);
  MotorolaFileIO& operator>> (signed short& s)
    {  return *this >> (unsigned short&)s;  }
  MotorolaFileIO& _Export operator>> (unsigned long& l);
  MotorolaFileIO& operator>> (signed long& l)
    {  return *this>> (unsigned long&)l;  }
  MotorolaFileIO& _Export operator>> (MString& string);

  // Erstes Zeichen angucken
  unsigned char _Export peek ();

  // positionieren
  void _Export seek (const long int position);
  long int _Export tell (void);
};

#endif
