/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: NotifyingList.hpp                                                 *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the template class:                                         *
*     NotifyingList                                                            *
*******************************************************************************/
#ifndef _NOTIFYINGLIST_
#define _NOTIFYINGLIST_

#include <stl.h>
#include <Notifier.hpp>
#include <Observer.hpp>

template <class T>
class NotifyingList : public list<T>, public Notifier {
public:
  enum { removeElement = 0, removeAllElements = 1 };

protected:
  class _iterator : public iterator {
  public:
    _iterator(const iterator &source)
      : iterator(source) {}
    link_type& getnode() { return node; }
  };

public:
  class ObservingIterator : public iterator, public Observer {
  public:
    ObservingIterator(NotifyingList<T>& list)
      : iterator(), Observer(list) {}
    ObservingIterator(const ObservingIterator& source)
      : iterator(source), Observer(source) {}
    ObservingIterator(NotifyingList<T>& list, const iterator& iter)
      : iterator(iter), Observer(list) {}

    ObservingIterator& operator=(const ObservingIterator& source) {
      Observer::operator=(source);
      node = source.node;
      return *this;
    }

    ObservingIterator& operator++() {
      return (ObservingIterator&) iterator::operator++();
    }

    ObservingIterator operator++(int) {
      ObservingIterator tmp(*this);
      ++*this;
      return tmp;
    }

    ObservingIterator& operator--() {
      return (ObservingIterator&) iterator::operator--();
    }

    ObservingIterator operator--(int) {
      ObservingIterator tmp(*this);
      --*this;
      return tmp;
    }

    void handleNotification(int id, void* userData) {
      if ((id == removeAllElements) ||
          ((id == removeElement) && (node == * (link_type*) userData)))
        * (iterator*) this = ((NotifyingList<T>*) notifier)->end();
    }
  };

  ObservingIterator obegin() {
    return ObservingIterator(*this, begin());
  }

  ObservingIterator oend() {
    return ObservingIterator(*this, end());
  }

  void erase(iterator position) {
    if (position != end()) {
      list<T>::erase(position);
      notifyObservers(removeElement, &_iterator(position).getnode());
    }
  }

  void erase(iterator first, iterator last) {
    while (first != last) erase(first++);
  }

  NotifyingList()
    : list<T>(), Notifier() {}
  NotifyingList(const list<T>& source)
    : list<T>(source), Notifier() {}

  NotifyingList<T>& operator=(const list<T>& source) {
    if (&source != this) {
      notifyObservers(removeAllElements, 0);
      return (NotifyingList<T>&) list<T>::operator=(source);
    } else
      return *this;
  }

  NotifyingList<T>& operator+=(const list<T>& list) {
    insert(end(), list.begin(), list.end());
    return *this;
  }

  NotifyingList<T>& operator+=(const T& t) {
    insert(end(), t);
    return *this;
  }

};

#endif
