/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: NotifyingPointerList.hpp                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the template class:                                         *
*     NotifyingPointerList                                                     *
*******************************************************************************/
#ifndef _NOTIFYINGPOINTERLIST_
#define _NOTIFYINGPOINTERLIST_

#include <NotifyingList.hpp>
#include <Observer.hpp>

template <class T>
class NotifyingPointerList : public NotifyingList<T*> {
public:
  iterator insert(iterator position, T *const x) {
    return NotifyingList<T*>::insert(position, x);
  }

  iterator insert(iterator position, const T& x) {
    return insert(position, x.copy ());
  }

  void insert(iterator position, const_iterator first, const_iterator last) {
    while (first != last) insert(position, **first++);
  }

  void erase(iterator position) {
    if (position != end()) {
      delete *position;
      NotifyingList<T*>::erase(position);
    }
  }

  void erase(iterator first, iterator last) {
    while (first != last) erase(first++);
  }

  NotifyingPointerList()
    : NotifyingList<T*>() {}
  NotifyingPointerList(const list<T*>& source)
    : NotifyingList<T*>() {
    insert(end(), source.begin(), source.end());
  }

  NotifyingPointerList<T>& operator=(const list<T*>& source) {
    erase(begin(), end());
    insert(end(), source.begin(), source.end());
    return *this;
  }

  NotifyingPointerList<T>& operator+=(const list<T*>& list) {
    insert(end(), list.begin(), list.end());
    return *this;
  }

  NotifyingPointerList<T>& operator+=(T *const t) {
    insert(end(), t);
    return *this;
  }
};

#endif
