/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: Observer.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     Observer                                                                 *
*******************************************************************************/
#ifndef _OBSERVER_
#define _OBSERVER_

class Notifier;

class Observer {
  friend class Notifier;

protected:
  Notifier *notifier;

private:
  void handleDestroyNotification() {
    notifier = 0;
  }

protected:
  virtual void _Export handleNotification(int id, void *userData) = 0;

public:
  // Constructor
  inline Observer(Notifier& notifierTemp);
  inline Observer(const Observer &source);

  inline Observer& operator=(const Observer &source);

  // Destructor
  virtual _Export ~Observer();
};

#include <Notifier.hpp>

inline Observer::Observer(Notifier &notifierTemp) {
  notifier = &notifierTemp;
  notifier->registerObserver(*this);
}

inline Observer::Observer(const Observer &source) {
  notifier = source.notifier;
  notifier->registerObserver(*this);
}

inline Observer& Observer::operator=(const Observer &source) {
  if (notifier != source.notifier) {
    notifier->deregisterObserver(*this);
    notifier = source.notifier;
    notifier->registerObserver(*this);
  }
  return *this;
}

#endif
