/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: TextFileIO.cpp                                                    *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     TextFileIO                                                               *
*******************************************************************************/

#include <TextFileIO.hpp>

// ffnen und schliessen einer Textdatei
void TextFileIO::open (const MString& name, const OpenMode mode)
{
  switch (mode)
    {
      case readOnly:
           file = fopen (name, "r");
           break;
      case writeOnly:
           file = fopen (name, "w");
           break;
      case append:
           file = fopen (name, "a");
           break;
      case readWrite:
           file = fopen (name, "r+");
           break;
      case writeRead:
           file = fopen (name, "w+");
           break;
      case appendRead:
           file = fopen (name, "a+");
           break;
    }

  if (file == NULL)
    throw (Exception (ERROR_FILE_OPEN));
}

void TextFileIO::open (IOStream stream)
{
  switch (stream)
    {
      case stdIn:
	   file = stdin;
	   break;
      case stdOut:
	   file = stdout;
	   break;
      case stdErr:
           file = stderr;
	   break;
    }
}

void TextFileIO::close ()
{
  if ((file != stdin)&&(file != stdout)&&
      (file != stderr)&&(file != NULL))
    fclose (file);
  file = NULL;
}

// Schreiben in eine Datei
TextFileIO& TextFileIO::operator<< (const unsigned char c)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  if (EOF == fputc (c, file))
    throw Exception (ERROR_FILE_WRITE);

  return *this;
}

TextFileIO& TextFileIO::operator<< (const MString& string)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  if (EOF == fputs (string, file))
    throw Exception (ERROR_FILE_WRITE);

  return *this;
}

// lesen aus TextFile
TextFileIO& TextFileIO::operator>> (unsigned char& c)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  int i;
  if (EOF == (i = fgetc (file)))
    throw Exception (ERROR_FILE_READ);

  c = (unsigned char)i;

  return *this;
}

TextFileIO& TextFileIO::operator>> (unsigned long& l)
{
  MString string;
  *this >> string;
  l = string.asUnsigned ();
  return *this;
}

TextFileIO& TextFileIO::operator>> (signed long& l)
{
  MString string;
  *this >> string;
  l = string.asSigned ();
  return *this;
}

TextFileIO& TextFileIO::operator>> (MString& string)
{
  if (file == NULL)
    throw (Exception (ERROR_FILE_NOT_OPEN));

  char buffer[256];
  if (NULL == fgets (buffer, sizeof (buffer), file))
    throw Exception (ERROR_FILE_WRITE);
  string = buffer;
  return *this;
}

