/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: TextFileIO.hpp                                                    *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     TextFileIO                                                               *
*******************************************************************************/
#ifndef _TEXTFILEIO_
#define _TEXTFILEIO_

#include <FileIO.hpp>
#include <Exception.hpp>

class TextFileIO : public FileIO
{
protected:
  FILE *file;

public:
  enum IOStream {stdIn, stdOut, stdErr};

  // Constructor / Destructor
  TextFileIO ()
    {  file = NULL;  }
  TextFileIO (const MString& name, const OpenMode mode)
    {  open (name, mode);  }
  TextFileIO (const IOStream stream)
    {  open (stream);  }
  virtual ~TextFileIO ()
    {  close ();  }

  // ffnen und schliessen einer Textdatei
  void _Export open (const MString& name, const OpenMode mode);
  void _Export open (IOStream stream);
  void _Export close ();

  // Schreiben in eine Datei
  TextFileIO& _Export operator<< (const unsigned char c);
  TextFileIO& operator<< (const signed char c)
    {  return *this << (unsigned char)c;  }
  TextFileIO& operator<< (const unsigned short s)
    {  return *this << (unsigned long)s;  }
  TextFileIO& operator<< (const signed short s)
    {  return *this << (signed long)s;  }
  TextFileIO& operator<< (const unsigned long l)
    {  return *this << MString (l);  }
  TextFileIO& operator<< (const signed long l)
    {  return *this << MString (l);  }
  TextFileIO& _Export operator<< (const MString& string);

  // lesen aus TextFile
  TextFileIO& _Export operator>> (unsigned char& c);
  TextFileIO& operator>> (signed char& c)
    {  unsigned char uc; *this >> uc;
       c = (signed char)uc;  return *this;  }
  TextFileIO& operator>> (unsigned short& s)
    {  unsigned long l; *this >> l;
       s = (unsigned short)l; return *this;  }
  TextFileIO& operator>> (signed short& s)
    {  signed long l; *this >> l;
       s = (signed short)l;  return *this;  }
  TextFileIO& _Export operator>> (unsigned long& l);
  TextFileIO& _Export operator>> (signed long& l);
  TextFileIO& _Export operator>> (MString& string);

  // Erstes Zeichen angucken
  unsigned char peek ()
    {  throw Exception (ERROR_FILE_READ);  }

  // positionieren
  void seek (const long int)
    {  throw Exception (ERROR_FILE_SEEK);  }
  long int tell (void)
    {  throw Exception (ERROR_FILE_TELL);  }
};

#endif
