/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: Midi.cpp                                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     Midi                                                                     *
*******************************************************************************/
#include "Exception.hpp"
#include "Midi.hpp"

#include "MidiErrors.h"

// Convertiert Midi nach Typ 0 und gibt Zeiger auf neuen Track zurck
MidiTrackChunk* Midi::convertToType0 () const
{
  // Wenn schon in Format 0, fertig
  if (header.format () == 0)
    return 0;

  // Wenn keine Tracks vorhanden, fertig
  if (empty ())
    return 0;

  // TrackCursor erzeugen
  const_iterator iter = begin ();

  // Tracks zu einem zusammenfassen
  MidiTrackChunk *midiTrackChunk = new MidiTrackChunk ();

  // Wenn Typ 1 Midi Tracks mit oder verknpfen
  if (header.format () == 1)
    while (iter != end ())
      {
        *midiTrackChunk |= **iter;
        ++iter;
      }

  // Wenn Typ 2 Midi Tracks aneinander haengen
  if (header.format () == 2)
    while (iter != end ())
      {
        *midiTrackChunk += **iter;
        ++iter;
      }

  return midiTrackChunk;
}

// Convertiert Midi nach Typ 0
void Midi::convertToType0 ()
{
  // Wenn schon in Format 0, fertig
  if (header.format () == 0)
    return ;

  // Wenn keine Tracks vorhanden, fertig
  if (empty ())
    return ;

  // Tracks zu einem zusammenfassen
  MidiTrackChunk *midiTrackChunk = ((const Midi*)this)->convertToType0 ();

  erase (begin (), end ());

  *this += midiTrackChunk;
  header.setFormat (0);
}

// Stcklnge in mSekunden erfragen
unsigned long Midi::length () const
{
  // Wenn Sequence leer, 0 zurckgeben
  if (empty ())
    return (0);

  // Stcklnge ausrechnen
  unsigned short ticksPerQuarter = header.division () & 0x7FFF;

  switch (header.format ())
    {
      case 0:
           // Lnge des ersten Tracks erfragen
           return (front ())->length (ticksPerQuarter);
      case 1:
           {
             // Maximale Tracklnge erfragen
             unsigned long laenge, laengeTemp;
             laenge = 0;

             const_iterator iter = begin ();

             while (iter != end ())
               if (laenge < (laengeTemp = (*(iter++))->length (ticksPerQuarter)))
                 laenge = laengeTemp;

             return laenge;
           }
      case 2:
           {
             // Gesamtlnge aller Tracks errechnen
             unsigned long laenge = 0;

             const_iterator iter = begin ();

             while (iter != end ())
               laenge += (*(iter++))->length (ticksPerQuarter);

             return laenge;
           }
    }
  return 0;
}

// Titel und Komponist erfragen
MString Midi::titel () const
{
  if (empty ())
    return ("");
  return (front ())->titel ();
}

MString Midi::copyright () const
{
  if (empty ())
    return ("");
  return (front ())->copyright ();
}

// Laden einer Midi-Datei
FileIO& operator>> (FileIO& fileIO, Midi& midi)
{
  unsigned char c, c2, c3, c4;

  // Kennung berprfen
  fileIO >> c >> c2 >> c3 >> c4;
  if ((c != 'M')||(c2 != 'T')||(c3 != 'h')||(c4 != 'd'))
    throw (Exception (ERROR_MIDI_HEADER_CHUNK));

  // Lnge berprfen
  unsigned long l;
  fileIO >> l;
  if (l != 6)
    throw (Exception (ERROR_MIDI_HEADER_CHUNK));

  // Format einlesen
  unsigned short s, trackCount;
  fileIO >> s;
  midi.header.setFormat (s);

  // TrackCount einlesen
  fileIO >> trackCount;

  // Division einlesen, Fehler wenn oberstes Bit gesetzt
  fileIO >> s;
  if (s & 0x8000)
    throw (Exception (ERROR_MIDI_DIVISION));
  midi.header.setDivision (s);

  // TrackChunk lesen
  while (trackCount-- > 0)
    {
      MidiTrackChunk *midiTrackChunk = new MidiTrackChunk ();
      fileIO >> *midiTrackChunk;
      midi += midiTrackChunk;
    }

  return fileIO;
}

// Sichern einer Midi-Datei
FileIO& _Export operator<< (FileIO& fileIO, const Midi& midi)
{
  fileIO << (unsigned char)'M' << (unsigned char)'T'
           << (unsigned char)'h' << (unsigned char)'d';
  fileIO << (unsigned long)6;
  fileIO << midi.header.format ();
  fileIO << (unsigned short)midi.trackCount ();
  fileIO << midi.header.division ();

  // Tracks sichern
  TrackList::const_iterator iter = midi.begin ();

  while (iter != midi.end ())
    fileIO << **(iter++);

  return fileIO;
}




