/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiChannelPressure.cpp                                           *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiChannelPressure                                                      *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiChannelPressure.hpp"

#include "MidiErrors.h"

// Message kopieren
MidiMessage* MidiChannelPressure::copy () const
{
  MidiChannelPressure *messageTemp = new MidiChannelPressure ();
  messageTemp->setDelay (delay ());
  messageTemp->setChannel (channel ());
  messageTemp->setPressure (pressure ());
  return (messageTemp);
}

// Pressure setzen und berprfen
void MidiChannelPressure::setPressure (const unsigned char pressureTemp)
{
  if (pressureTemp > 0x7F)
    throw (Exception (ERROR_MIDI_PRESSURE));

  midiPressure = pressureTemp;
}

// sichern
IO& MidiChannelPressure::out (IO& iO) const
{
  return iO << (unsigned char)(0xD0 | channel ())
                << pressure ();
}

// laden und ueberpruefen
IO& MidiChannelPressure::in (IO& iO)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde und, da der Kanal
  // nachtrglich gesetzt wird
  iO >> c;
  setPressure (c);

  return iO;
}

