/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiControlChange.cpp                                             *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiControlChange                                                        *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiControlChange.hpp"

#include "MidiErrors.h"

// Constructor
MidiControlChange::MidiControlChange (void)
  : MidiChannelMessage ()
{
  midiControl = 0;
  midiValue = 0;
}

// Message kopieren
MidiMessage* MidiControlChange::copy () const
{
  MidiControlChange *messageTemp = new MidiControlChange;
  messageTemp->setDelay (delay ());
  messageTemp->setChannel (channel ());
  messageTemp->setControl (control ());
  messageTemp->setValue (value ());
  return messageTemp;
}

// Control setzen und ueberpruefen
void MidiControlChange::setControl (const unsigned char controlTemp)
{
  if (controlTemp > 0x7F)
    throw (Exception (ERROR_MIDI_DATA));

  midiControl = controlTemp;
}

// Value setzen und ueberpruefen
void MidiControlChange::setValue (const unsigned char valueTemp)
{
  if (valueTemp > 0x7F)
    throw (Exception (ERROR_MIDI_DATA));

  midiValue = valueTemp;
}

// Ausgabe
IO& MidiControlChange::out (IO& iO) const
{
  return iO << (unsigned char)(0xB0 | channel())
                << control () << value ();
}

// Einlesen und berprfen
IO& MidiControlChange::in (IO& iO)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde und, da der Kanal
  // nachtrglich gesetzt wird
  iO >> c;
  setControl (c);

  iO >> c;
  setValue (c);

  return iO;
}

