/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiDelay.cpp                                                     *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiDelay                                                                *
*******************************************************************************/
#include <Exception.hpp>

#include "MidiDelay.hpp"

#include "MidiErrors.h"

// Constructor mit ulong delay Angeabe
MidiDelay::MidiDelay (const unsigned long delayTemp)
{
  // Wenn Angabe zu hoch Fehler!
  if (delayTemp >> 28)
    throw (Exception (ERROR_MIDI_DELAY));

  timeDelay = delayTemp;
}

// Zuweisungsop. fuer MidiDelay
MidiDelay& MidiDelay::operator= (const MidiDelay& midiDelayTemp)
{
  timeDelay = midiDelayTemp.timeDelay;
  return *this;
}

// Zuweisungsop. fuer ulong delay
MidiDelay& MidiDelay::operator= (const unsigned long delayTemp)
{
  // Wenn Angabe zu hoch Fehler!
  if (delayTemp >> 28)
    throw (Exception (ERROR_MIDI_DELAY));

  timeDelay = delayTemp;
  return *this;
}

// += Operator
MidiDelay& MidiDelay::operator+= (const MidiDelay& midiDelayTemp)
{
  // Addieren
  timeDelay += midiDelayTemp.timeDelay;

  // Neuen Wert ueberpruefen
  if (timeDelay >> 28)
    throw (Exception (ERROR_MIDI_DELAY));

  return (*this);
}


// += Operator
MidiDelay& MidiDelay::operator+= (const long delayTemp)
{
  // Addieren
  timeDelay += delayTemp;

  // Neuen Wert ueberpruefen
  if (timeDelay >> 28)
    throw (Exception (ERROR_MIDI_DELAY));

  return (*this);
}

// ld eine MidiDelay Angabe aus FileIO
FileIO& operator>> (FileIO& fileIO, MidiDelay& midiDelay)
{
  unsigned char c;

  // timeDelay auf 0 initialisieren
  unsigned long timeDelay = 0;
  do
    {
      // erste Zahl holen
      fileIO >> c;

      // schon gelesenen Wert um 7 weiterschieben und die neuen 7 Bit addieren
      timeDelay <<= 7;
      timeDelay += c & 0x7F;
    }
  // Wenn das hoechste Bit gesetzt ist, folgt mindestens noch eine D-Time Angabe
  while (c & 0x80);

  // neue Zeit midiDelay zuweisen
  midiDelay = timeDelay;

  return fileIO;
}

// speichert eine MidiDelay Angabe in FileIO
FileIO& operator<< (FileIO& fileIO, const MidiDelay& midiDelay)
{
  // je nach Zahlengroesse 1-4 7Bit zahlen schreiben. Bei allen ausser der
  // letzten das hoechste 8 Bit setzen
  if (midiDelay & 0x0FE00000L)
    fileIO << (unsigned char)((midiDelay >> 21) | 0x80);
  if (midiDelay & 0x0FFFC000L)
    fileIO << (unsigned char)((midiDelay >> 14) | 0x80);
  if (midiDelay & 0x0FFFFF80L)
    fileIO << (unsigned char)((midiDelay >> 7) | 0x80);
  return fileIO << (unsigned char)(midiDelay & 0x7F);
}

