/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiKeyPressure.cpp                                               *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiKeyPressure                                                          *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiKeyPressure.hpp"

#include "MidiErrors.h"

// Constructor
MidiKeyPressure::MidiKeyPressure (void)
  : MidiChannelMessage ()
{
  midiKey = 0;
  midiDynamic = 0;
}

// Message kopieren
MidiMessage* MidiKeyPressure::copy () const
{
  MidiKeyPressure *messageTemp = new MidiKeyPressure ();
  messageTemp->setDelay (delay());
  messageTemp->setChannel (channel ());
  messageTemp->setKey (key ());
  messageTemp->setDynamic (dynamic ());
  return (messageTemp);
}

// Tonhoehe setzen und ueberpruefen
void MidiKeyPressure::setKey (const unsigned char keyTemp)
{
  if (keyTemp > 0x7F)
    throw (Exception (ERROR_MIDI_KEY));

  midiKey = keyTemp;
}

// Tonhhe setzen und berprfen
void MidiKeyPressure::setDynamic (const unsigned char dynamicTemp)
{
  if (dynamicTemp > 0x7F)
    throw (Exception (ERROR_MIDI_DYNAMIC));

  midiDynamic = dynamicTemp;
}

// Ausgeben
IO& MidiKeyPressure::out (IO& iO) const
{
  return iO << (unsigned char)(0xA0 | channel())
                << key () << dynamic ();
}

// Einlesen und berprfen
IO& MidiKeyPressure::in (IO& iO)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde und, da der Kanal
  // nachtrglich gesetzt wird
  iO >> c;
  setKey (c);

  iO >> c;
  setDynamic (c);

  return iO;
}

