/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiMessage.hpp                                                   *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     MidiMessage                                                              *
*******************************************************************************/
#ifndef _MIDIMESSAGE_
#define _MIDIMESSAGE_

#include <Message.hpp>
#include <MidiDelay.hpp>
#include <SequencerIO.hpp>
#include <FileIO.hpp>

class MidiMessage : public Message
{
protected:
  // Verzgerung zur vorigen Message
  MidiDelay midiDelay;

public:
  // Verzgerung einer MidiMessage
  MidiDelay delay (void) const
    {  return midiDelay;  }
  void setDelay (const MidiDelay& midiDelayTemp)
    {  midiDelay = midiDelayTemp;  }

  // copieren von Messages
  virtual MidiMessage* _Export copy () const = 0;

  // IO Ein und Ausgabe
  virtual IO& _Export out (IO& iO) const = 0;
  virtual IO& _Export in (IO& iO) = 0;
  virtual SequencerIO& sequencerOut (SequencerIO& sequencerIO) const
    {  return ((SequencerIO&)out (sequencerIO));  }
  virtual SequencerIO& sequencerIn (SequencerIO& sequencerIO)
    {  return ((SequencerIO&)in (sequencerIO));  }
  virtual FileIO& fileOut (FileIO& fileIO) const
    {  return ((FileIO&)out (fileIO));  }
  virtual FileIO& fileIn (FileIO& fileIO)
    {  return ((FileIO&)in (fileIO));  }
};

// IO Ein und Ausgabe operatoren
inline SequencerIO& operator<< (SequencerIO& sequencerIO, const MidiMessage& midiMessage)
  {  return midiMessage.sequencerOut (sequencerIO); }
inline SequencerIO& operator>> (SequencerIO& sequencerIO, MidiMessage& midiMessage)
  {  return midiMessage.sequencerIn (sequencerIO);  }
inline FileIO& operator<< (FileIO& fileIO, const MidiMessage& midiMessage)
  {  return midiMessage.fileOut (fileIO); }
inline FileIO& operator>> (FileIO& fileIO, MidiMessage& midiMessage)
  {  return midiMessage.fileIn (fileIO);  }

#endif

