/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiMetaEvent.cpp                                                 *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiMetaEvent                                                            *
*******************************************************************************/

#include <stdlib.h>
#include <string.h>

#include "MidiMetaEvent.hpp"

#include "MidiErrors.h"

// Constructor
MidiMetaEvent::MidiMetaEvent (void)
  : MidiFileMessage ()
{
  metaId = 0;
  metaLaenge = 0;
  metaContent = 0;
}

// Message kopieren
MidiMessage* MidiMetaEvent::copy () const
{
  MidiMetaEvent *messageTemp = new MidiMetaEvent ();
  messageTemp->setDelay (delay());
  messageTemp->setMetaEventId (metaEventId ());
  messageTemp->setContent (metaContent, metaLaenge);
  return (messageTemp);
}

// Content setzen
void MidiMetaEvent::setContent (const MidiMetaEvent& midiMetaEventTemp)
{
  if (metaContent != 0)
    delete[] metaContent;

  metaLaenge = midiMetaEventTemp.content (&metaContent);
}

// Content holen
MidiDelay MidiMetaEvent::content (unsigned char *contentTemp[]) const
{
  // Wenn kontent leer, 0 zurueckgeben
  if (metaLaenge == 0)
    return 0;

  // content nach contentTemp kopieren
  *contentTemp = new unsigned char [metaLaenge];
  memcpy (*contentTemp, metaContent, sizeof (unsigned char) * metaLaenge);

  return metaLaenge;
}

// Content setzen
void MidiMetaEvent::setContent (const unsigned char *contentTemp, const MidiDelay& laengeTemp)
{
  // Vorigen Inhalt loeschen
  if (metaContent != 0)
    delete[] metaContent;
  metaContent = 0;
  metaLaenge + 0;

  // Wenn neuer content leer, dann fertig
  if (laengeTemp == 0)
    return;

  // neuen content kopieren
  metaContent = new unsigned char [laengeTemp];
  memcpy (metaContent, contentTemp, sizeof (unsigned char) * laengeTemp);

  metaLaenge = laengeTemp;
}

// Content holen
MString MidiMetaEvent::content () const
{
  char *contentTemp = new char [metaLaenge + 1];
  memcpy (contentTemp, metaContent, sizeof (char) * metaLaenge);
  contentTemp[metaLaenge] = '\0';

  MString string (contentTemp);
  delete[] contentTemp;

  return string;
}

// Content setzen
void MidiMetaEvent::setContent (const MString& string)
{
  if (metaContent != 0)
    delete[] metaContent;

  metaContent = new unsigned char [string.length ()];
  memcpy (metaContent, (char*)string, sizeof (char) * string.length ());

  metaLaenge = string.length ();
}

// MidiMetaEvent sichern
FileIO& MidiMetaEvent::fileOut (FileIO& fileIO) const
{
  unsigned long l;
  unsigned char *contentTemp;
  MidiDelay laengeTemp;

  fileIO << (unsigned char)(0xFF) << metaEventId ()
           << (laengeTemp = content (&contentTemp));

  for (l = 0; l < laengeTemp; l++)
    fileIO << contentTemp [l];

  if (laengeTemp > 0)
    delete[] contentTemp;

  return fileIO;
}

// MidiMetaEvent laden
FileIO& MidiMetaEvent::fileIn (FileIO& fileIO)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde
  fileIO >> c;
  setMetaEventId (c);

  MidiDelay laengeTemp;
  fileIO >> laengeTemp;

  unsigned char *contentTemp = new unsigned char [laengeTemp];
  unsigned long l;
  for (l = 0; l < laengeTemp; l++)
    fileIO >> contentTemp [l];

  setContent (contentTemp, laengeTemp);
  if (laengeTemp > 0)
    delete[] contentTemp;

  return fileIO;
}

// MidiSequencer Ausgabe
SequencerIO& MidiMetaEvent::sequencerOut (SequencerIO&) const
{
  throw (Exception (ERROR_MIDI_META_EVENT_IO));
}

// Constructor
MidiMetaSetTempo::MidiMetaSetTempo ()
  : MidiMetaEvent ()
{
  unsigned char buffer[3] = {0,0,0};
  setMetaEventId (META_EVENT_SET_TEMPO);
  setContent (buffer, 3);
}

// Message kopieren
MidiMessage* MidiMetaSetTempo::copy () const
{
  MidiMetaEvent *messageTemp = new MidiMetaSetTempo ();
  messageTemp->setDelay (delay());
  messageTemp->setMetaEventId (metaEventId ());
  messageTemp->setContent (metaContent, metaLaenge);
  return (messageTemp);
}

// Tempo holen
unsigned long MidiMetaSetTempo::tempo () const
{
  if (metaLaenge < 3)
    throw (Exception (ERROR_MIDI_NO_META_SET_TIME_CONTENT));

  unsigned long tempoTemp = (metaContent[0] << 16) + (metaContent[1] << 8) + metaContent[2];
  return tempoTemp;
}

// Tempo setzen
void MidiMetaSetTempo::setTempo (const unsigned long tempoTemp)
{
  unsigned char buffer[3];
  buffer[0] = (tempoTemp >> 16) & 0xFF;
  buffer[1] = (tempoTemp >> 8) & 0xFF;
  buffer[2] = tempoTemp & 0xFF;
  setContent (buffer, 3);
}

// Constructor
MidiMetaEndOfTrack::MidiMetaEndOfTrack ()
  : MidiMetaEvent ()
{
  setMetaEventId (META_EVENT_END_OF_TRACK);
  setContent (0, 0);
}

// Message kopieren
MidiMessage* MidiMetaEndOfTrack::copy () const
{
  MidiMetaEvent *messageTemp = new MidiMetaEndOfTrack ();
  messageTemp->setDelay (delay());
  messageTemp->setMetaEventId (metaEventId ());
  messageTemp->setContent (metaContent, metaLaenge);
  return (messageTemp);
}

// MetaEvents betimmen
MidiMetaEvent* _Export bestimmeMidiMetaEvent (IO& io)
{
  switch (io.peek ())
    {
      case META_EVENT_END_OF_TRACK:
           return new MidiMetaEndOfTrack ();
      case META_EVENT_SET_TEMPO:
           return new MidiMetaSetTempo ();
      default:
           return new MidiMetaEvent ();
    }
}
