/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiMetaEvent.hpp                                                 *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     MidiMetaEvent                                                            *
*******************************************************************************/
#ifndef _MIDIMETAEVENT_
#define _MIDIMETAEVENT_

#include <MidiFileMessage.hpp>

// MetaEvent Ids
#define META_EVENT_SEQUENCE_NUMBER 0x00
#define META_EVENT_TEXT            0x01
#define META_EVENT_COPYRIGHT       0x02
#define META_EVENT_TRACK_NAME      0x03
#define META_EVENT_INSTRUMENT_NAME 0x04
#define META_EVENT_LYRICS          0x05
#define META_EVENT_MARKER          0x06
#define META_EVENT_CUE             0x07
#define META_EVENT_MIDI_CHANNEL_PREFIX 0x20
#define META_EVENT_END_OF_TRACK    0x2F
#define META_EVENT_SET_TEMPO       0x51
#define META_EVENT_SMPTE           0x54
#define META_EVENT_TIME_SIGNATURE  0x58
#define META_EVENT_KEY_SIGNATURE   0x59
#define META_EVENT_SEQUENCER_SPECIFIC 0x7F

class MidiMetaEvent : public MidiFileMessage
{
protected:
  unsigned char metaId;
  MidiDelay metaLaenge;
  unsigned char *metaContent;

public:
  // Constructor
  _Export MidiMetaEvent (void);

  // Message kopieren
  MidiMessage* _Export copy () const;

  // Message ID
  unsigned char id () const
    {  return 0xFF;  }

  // Id
  unsigned char metaEventId () const
    {  return metaId;  }
  void setMetaEventId (const unsigned char idTemp)
    {  metaId = idTemp; }

  // Content
  void _Export setContent (const MidiMetaEvent& midiMetaEventTemp);
  MidiDelay _Export content (unsigned char *contentTemp[]) const;
  void _Export setContent (const unsigned char *contentTemp, const MidiDelay& laengeTemp);
  MString _Export content () const;
  void _Export setContent (const MString& string);

  // sichern/laden
  FileIO& _Export fileOut (FileIO& fileIO) const;
  FileIO& _Export fileIn (FileIO& fileIO);

  // Ein und Ausgabe
  SequencerIO& _Export sequencerOut (SequencerIO& sequencerIO) const;
};

class MidiMetaSetTempo : public MidiMetaEvent
{
public:
  // Constructor
  _Export MidiMetaSetTempo ();

  // Message kopieren
  MidiMessage* _Export copy () const;

  // Tempo usec per Quater
  unsigned long _Export tempo () const;
  void _Export setTempo (const unsigned long tempoTemp);
};

class MidiMetaEndOfTrack : public MidiMetaEvent
{
public:
  // Constructor
  _Export MidiMetaEndOfTrack ();

  // Message kopieren
  MidiMessage* _Export copy () const;
};

// MetaEvents betimmen
MidiMetaEvent* _Export bestimmeMidiMetaEvent (IO& io);

#endif

