/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiNoteOff.cpp                                                   *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiNoteOff                                                              *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiNoteOff.hpp"

#include "MidiErrors.h"

// Constructor
MidiNoteOff::MidiNoteOff (void)
  : MidiChannelMessage ()
{
  midiKey = 0;
  midiDynamic = 0;
}

// Message kopieren
MidiMessage* MidiNoteOff::copy () const
{
  MidiNoteOff *messageTemp = new MidiNoteOff ();
  messageTemp->setDelay (delay());
  messageTemp->setChannel (channel ());
  messageTemp->setKey (key ());
  messageTemp->setDynamic (dynamic ());
  return (messageTemp);
}

// Tonhoehe setzen und ueberpruefen
void MidiNoteOff::setKey (const unsigned char keyTemp)
{
  if (keyTemp > 0x7F)
    throw (Exception (ERROR_MIDI_KEY));

  midiKey = keyTemp;
}

// Tonhoehe setzen und ueberpruefen
void MidiNoteOff::setDynamic (const unsigned char dynamicTemp)
{
  if (dynamicTemp > 0x7F)
    throw (Exception (ERROR_MIDI_DYNAMIC));

  midiDynamic = dynamicTemp;
}

// MidiNoteOff ausgeben
IO& MidiNoteOff::out (IO& iO) const
{
  return iO << (unsigned char)(0x80 | channel())
                << key () << dynamic ();
}

// MidiNoteOff einlesen und berprfen
IO& MidiNoteOff::in (IO& iO)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde und, da der Kanal
  // nachtrglich gesetzt wird
  iO >> c;
  setKey (c);

  iO >> c;
  setDynamic (c);

  return iO;
}

