/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiNoteOn.cpp                                                    *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiNoteOn                                                               *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiNoteOn.hpp"

#include "MidiErrors.h"

// Constructor
MidiNoteOn::MidiNoteOn (void)
  : MidiChannelMessage ()
{
  midiKey = 0;
  midiDynamic = 0;
}

// Message kopieren
MidiMessage* MidiNoteOn::copy () const
{
  MidiNoteOn *messageTemp = new MidiNoteOn ();
  messageTemp->setDelay (delay());
  messageTemp->setChannel (channel ());
  messageTemp->setKey (key ());
  messageTemp->setDynamic (dynamic ());
  return (messageTemp);
}

// Tonhoehe setzen und ueberpruefen
void MidiNoteOn::setKey (const unsigned char keyTemp)
{
  if (keyTemp > 0x7F)
    throw (Exception (ERROR_MIDI_KEY));

  midiKey = keyTemp;
}

// Tonhoehe setzen und ueberpruefen
void MidiNoteOn::setDynamic (const unsigned char dynamicTemp)
{
  if (dynamicTemp > 0x7F)
    throw (Exception (ERROR_MIDI_DYNAMIC));

  midiDynamic = dynamicTemp;
}

// MidiNoteOn Ausgabe
IO& MidiNoteOn::out (IO& iO) const
{
  return iO << (unsigned char)(0x90 | channel())
                << key () << dynamic ();
}

// MidiNoteOn einlesen und berprfen
IO& MidiNoteOn::in (IO& iO)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde und, da der Kanal
  // nachtrglich gesetzt wird
  iO >> c;
  setKey (c);

  iO >> c;
  setDynamic (c);

  return iO;
}

