/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiPitchBendChange.cpp                                           *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiPitchBendChange                                                      *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiPitchBendChange.hpp"

#include "MidiErrors.h"

// Message kopieren
MidiMessage* MidiPitchBendChange::copy () const
{
  MidiPitchBendChange *messageTemp = new MidiPitchBendChange ();
  messageTemp->setDelay (delay());
  messageTemp->setChannel (channel ());
  messageTemp->setPitchBend (pitchBend ());
  return (messageTemp);
}

// Pitch Bend setzen und ueberpruefen
void MidiPitchBendChange::setPitchBend (const unsigned short pitchBendTemp)
{
  if (pitchBendTemp > 0x3FFF)
    throw (Exception (ERROR_MIDI_PITCH_BEND));

  midiPitchBend = pitchBendTemp;
}

// Ausgeben
IO& MidiPitchBendChange::out (IO& iO) const
{
  return iO << (unsigned char)(0xE0 | channel ())
                << (unsigned char)(pitchBend () & 0x7F)
                << (unsigned char)(pitchBend () >> 7);
}

// Einlesen und berprfen
IO& MidiPitchBendChange::in (IO& iO)
{
  unsigned char c, c2;

  // Erwartet, da richtiges Statusbyte gelesen wurde und, da der Kanal
  // nachtrglich gesetzt wird
  iO >> c >> c2;
  setPitchBend ((c2 << 7) + c);

  return iO;
}

