/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiProgramChange.cpp                                             *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiProgramChange                                                        *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiProgramChange.hpp"

#include "MidiErrors.h"

// Message kopieren
MidiMessage* MidiProgramChange::copy () const
{
  MidiProgramChange *messageTemp = new MidiProgramChange ();
  messageTemp->setDelay (delay());
  messageTemp->setChannel (channel ());
  messageTemp->setProgram (program ());
  return (messageTemp);
}

// Program setzen und ueberpruefen
void MidiProgramChange::setProgram (const unsigned char programTemp)
{
  if (programTemp > 0x7F)
    throw (Exception (ERROR_MIDI_PROGRAM));

  midiProgram = programTemp;
}

// Ausgabe
IO& MidiProgramChange::out (IO& iO) const
{
  return iO << (unsigned char)(0xC0 | channel ())
                << program ();
}

// Einlesen und berprfen
IO& MidiProgramChange::in (IO& iO)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde und, da der Kanal
  // nachtrglich gesetzt wird
  iO >> c;
  setProgram (c);

  return iO;
}
