/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiRealTimeMessages.cpp                                          *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiTimingClock                                                          *
*     MidiStart                                                                *
*     MidiContinue                                                             *
*     MidiStop                                                                 *
*     MidiActiveSensing                                                        *
*     MidiSystemReset                                                          *
*******************************************************************************/

#include "MidiRealTimeMessages.hpp"

#include "MidiErrors.h"

// Message kopieren
MidiMessage* MidiTimingClock::copy () const
{
  MidiTimingClock *messageTemp = new MidiTimingClock ();
  messageTemp->setDelay (delay());
  return (messageTemp);
}

// sichern
IO& MidiTimingClock::out (IO& iO) const
{
  return iO << (unsigned char)(0xF8);
}

// laden und ueberpruefen
IO& MidiTimingClock::in (IO& iO)
{
  // Erwartet, da richtiges Statusbyte gelesen wurde
  return iO;
}

// Message kopieren
MidiMessage* MidiStart::copy () const
{
  MidiStart *messageTemp = new MidiStart ();
  messageTemp->setDelay (delay());
  return (messageTemp);
}

// sichern
IO& MidiStart::out (IO& iO) const
{
  return iO << (unsigned char)(0xFA);
}

// laden und ueberpruefen
IO& MidiStart::in (IO& iO)
{
  // Erwartet, da richtiges Statusbyte gelesen wurde
  return iO;
}

// Message kopieren
MidiMessage* MidiContinue::copy () const
{
  MidiContinue *messageTemp = new MidiContinue ();
  messageTemp->setDelay (delay());
  return (messageTemp);
}

// sichern
IO& MidiContinue::out (IO& iO) const
{
  return iO << (unsigned char)(0xFB);
}

// laden und ueberpruefen
IO& MidiContinue::in (IO& iO)
{
  // Erwartet, da richtiges Statusbyte gelesen wurde
  return iO;
}

// Message kopieren
MidiMessage* MidiStop::copy () const
{
  MidiStop *messageTemp = new MidiStop ();
  messageTemp->setDelay (delay());
  return (messageTemp);
}

// sichern
IO& MidiStop::out (IO& iO) const
{
  return iO << (unsigned char)(0xFC);
}

// laden und ueberpruefen
IO& MidiStop::in (IO& iO)
{
  // Erwartet, da richtiges Statusbyte gelesen wurde
  return iO;
}

// Message kopieren
MidiMessage* MidiActiveSensing::copy () const
{
  MidiActiveSensing *messageTemp = new MidiActiveSensing ();
  messageTemp->setDelay (delay());
  return (messageTemp);
}

// sichern
IO& MidiActiveSensing::out (IO& iO) const
{
  return iO << (unsigned char)(0xFE);
}

// laden und ueberpruefen
IO& MidiActiveSensing::in (IO& iO)
{
  // Erwartet, da richtiges Statusbyte gelesen wurde
  return iO;
}

// Message kopieren
MidiMessage* MidiSystemReset::copy () const
{
  MidiSystemReset *messageTemp = new MidiSystemReset ();
  messageTemp->setDelay (delay());
  return (messageTemp);
}

// Ausgabe
SequencerIO& MidiSystemReset::sequencerOut (SequencerIO& sequencerIO) const
{
  sequencerIO << (unsigned char)(0xFF);
  return sequencerIO;
}

// Einlesen und berprfen
SequencerIO& MidiSystemReset::sequencerIn (SequencerIO& sequencerIO)
{
  unsigned char c;

  sequencerIO >> c;
  if (c != 0xFF)
    throw (Exception (ERROR_MIDI_REAL_TIME_MESSAGE));

  return sequencerIO;
}

