/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiSongPositionPointer.cpp                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiSongPositionPointer                                                  *
*******************************************************************************/
#include <Exception.hpp>
#include "MidiSongPositionPointer.hpp"

#include "MidiErrors.h"

// Message kopieren
MidiMessage* MidiSongPositionPointer::copy () const
{
  MidiSongPositionPointer *messageTemp = new MidiSongPositionPointer ();
  messageTemp->setDelay (delay());
  messageTemp->setPosition (position ());
  return (messageTemp);
}

// Position setzen und ueberpruefen
void MidiSongPositionPointer::setPosition (const unsigned short positionTemp)
{
  if (positionTemp > 0x3FFF)
    throw (Exception (ERROR_MIDI_POSITION));

  midiPosition = positionTemp;
}

// sichern
IO& MidiSongPositionPointer::out (IO& iO) const
{
  return iO << (unsigned char)(0xF2)
                << (unsigned char)(position () & 0x7F)
                << (unsigned char)(position () >> 7);
}

// laden und ueberpruefen
IO& MidiSongPositionPointer::in (IO& iO)
{
  unsigned char c, c2;

  // Erwartet, da richtiges Statusbyte gelesen wurde
  iO >> c >> c2;
  setPosition ((c2 << 7) + c);

  return iO;
}

