/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiSystemExclusive.cpp                                           *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Class implementation of the class:                                         *
*     MidiSystemExclusive                                                      *
*******************************************************************************/

#include <stdlib.h>

#include "MidiSystemExclusive.hpp"

#include "MidiErrors.h"

// Constructor
MidiSystemExclusive::MidiSystemExclusive (void)
{
  sysExId = 0;
  sysExLaenge = 0;
  sysExContent = 0;
}

// Message kopieren
MidiMessage* MidiSystemExclusive::copy () const
{
  MidiSystemExclusive *messageTemp = new MidiSystemExclusive ();
  messageTemp->setDelay (delay());
  messageTemp->setSystemExclusiveId (systemExclusivId ());
  messageTemp->setContent (sysExContent, sysExLaenge);
  return (messageTemp);
}

// Content setzen
void MidiSystemExclusive::setContent (const MidiSystemExclusive& midiSystemExclusiveTemp)
{
  if (sysExContent != 0)
    delete[] sysExContent;

  sysExLaenge = midiSystemExclusiveTemp.content (&sysExContent);
}

// Content holen
MidiDelay MidiSystemExclusive::content (unsigned char *contentTemp[]) const
{
  *contentTemp = new unsigned char [sysExLaenge];
  memcpy (*contentTemp, sysExContent, sizeof (unsigned char) * sysExLaenge);

  return sysExLaenge;
}

// Content setzen
void MidiSystemExclusive::setContent (const unsigned char *contentTemp, const MidiDelay& laengeTemp)
{
  if (sysExContent != 0)
    delete[] sysExContent;

  sysExContent = new unsigned char [laengeTemp];
  memcpy (sysExContent, contentTemp, sizeof (unsigned char) * laengeTemp);

  sysExLaenge = laengeTemp;
}

// MidiSystemExclusive sichern
IO& MidiSystemExclusive::out (IO& io) const
{
  unsigned long l;
  unsigned char *contentTemp;
  MidiDelay laengeTemp;

  io << (unsigned char)(0xF0) << systemExclusivId ();

  laengeTemp = content (&contentTemp);
  for (l = 0; l < laengeTemp; l++)
    io << contentTemp [l];
  delete[] contentTemp;

  // EOX schreiben
  io << (unsigned char)(0xF7);

  return io;
}

// MidiSystemExclusive laden
IO& MidiSystemExclusive::in (IO& io)
{
  unsigned char c;

  // Erwartet, da richtiges Statusbyte gelesen wurde
  io >> c;
  setSystemExclusiveId (c);

  unsigned long laengeTemp = 0;
  unsigned long laengeBuffer = 10;
  unsigned char *contentTemp = new unsigned char [laengeBuffer];
  for (io >> c; c != 0xF7; io >> c)
    {
      ++laengeTemp;
      if (laengeTemp > laengeBuffer)
        {
          unsigned char *buffer = contentTemp;
          contentTemp = new unsigned char [laengeBuffer *= 2];
          memcpy (contentTemp, buffer, (laengeTemp - 1) * sizeof (unsigned char));
          delete[] buffer;
        }

      io >> contentTemp[laengeTemp-1];
    }

  setContent (contentTemp, laengeTemp);
  delete[] contentTemp;

  return io;
}

