/*******************************************************************************
* Copyright Johannes Martin Software Entwicklung (L.Martin@TeamOS2.de)         *
********************************************************************************
* FILE NAME: MidiTrackChunk.hpp                                                *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class:                                                  *
*     MidiTrackChunk                                                           *
*******************************************************************************/
#ifndef _MIDITRACKCHUNK_
#define _MIDITRACKCHUNK_

#include <MidiMessage.hpp>
#include <NotifyingPointerList.hpp>

typedef NotifyingPointerList<MidiMessage> MidiMessageList;

class MidiTrackChunk : public MidiMessageList
{
protected:
  unsigned char lastStatusByte;  // Zuletzt gelesenes Statusbyte

public:
  // Constructor
  MidiTrackChunk (void)
    {  lastStatusByte = 0x90; } // MidiNoteOn
  MidiTrackChunk (const MidiTrackChunk& track)
    {  *this = track;  }

  // |= Operator
  MidiTrackChunk& _Export operator|= (const MidiTrackChunk& trackTemp);
  // += Operator
  MidiTrackChunk& operator+= (const MidiTrackChunk& trackTemp)
    {  MidiMessageList::operator+= (trackTemp);  return *this;  }
  MidiTrackChunk& operator+= (MidiMessage *message)
    {  MidiMessageList::operator+= (message);  return *this;  }
  // = Operator
  MidiTrackChunk& operator= (const MidiTrackChunk& track)
    {  MidiMessageList::operator= (track);
       lastStatusByte = track.lastStatusByte;
       return *this;  }
  MidiTrackChunk* copy () const
    {  MidiTrackChunk *track = new MidiTrackChunk;
       *track = *this;
       return track;  }

  // bestimmt die nchste MidiMessage und liefert einen Zeiger auf
  // die neu erzeugte Message zurck
  MidiMessage* _Export bestimmeMidiMessage (IO& io);

  // Tracklnge erfragen in msec
  unsigned long _Export length (const unsigned short ticksPerQuater) const;
  // Tracklnge erfragen in ticks
  unsigned long _Export length () const;

  // Titel und Komponist erfragen
  MString _Export titel () const;
  MString _Export copyright () const;
};

// sichern und laden
FileIO& _Export operator<< (FileIO& fileIO, const MidiTrackChunk& midiTrackChunk);
FileIO& _Export operator>> (FileIO& fileIO, MidiTrackChunk& midiTrackChunk);

#endif

